/*
  Copyright (C) MOXA Inc. All rights reserved.
  This software is distributed under the terms of the
  MOXA License.  See the file COPYING-MOXA for details.
*/

/*******************************************************************************
File name:	udp1.c
      -- UDP Server Client + Data Packing Example For firmware AP
      -- if receive data from serial port, send data to LAN with destination IP.
      -- if receive data from LAN, write data to serial port.

Description:
	This program create UDP socket and 
	exchange data between serial port and Ethernet.

	Use SOCKET Library,the header file sys/socket.h must be included.

	1	Open serial port.
	2	Configure serial port to 38400bps, N81, no flow control.
	3	Read remote IP address and UDP port settings.
	4	Create UDP socket and receive data on Port 4001. 
	5	Exchange data between serial port and Ethernet. 

Usage:
    	1. Make downloading file(udp1-release or udp1-debug) for UC7000 series 
    	    1.1 Create executable file(udp1-release or udp1-debug) from source code(udp1.c) 
    	    	1.1.1 Modify udp1.c: 
    	    		change "DEST_IP" following the PC Host's IP address.
    	    		change "UDPPORT" if needed.	
    	    	1.1.2 Compiler and link to make SDKudp.exe
	2. Download udp1-release or udp1-debug to UC7000 series 
     	    2.1 Using ftp download udp1-release or udp1-debug to a UC7000 series unit.    
    	3. Run Application on UC7000 series 
    	4. Execute UDP example for PC.[udpclient.exe or frmUdp.exe]
    	   input the UDP Port number equal to "UDPPORT"
    	         Destination ip equal to Network Enabler PCG's IP adddress.   
    	    
History:    
    	Version		Author		Date		Comment
        1.0		Victor Yu.	01-16-2004      Wrote it.

*******************************************************************************/
#include        <stdio.h>
#include        <stdlib.h>
#include        <sys/types.h>
#include        <sys/socket.h>
#include        <sys/fcntl.h>
#include        <termios.h>
#include        <netinet/in.h>
#include        <errno.h>

#define SPORT	"/dev/ttyM0"
#define UDPPORT	4001
#define DEST_IP	"192.168.4.140"


#define	DEL_CH		'b'
#define	BUF_LEN		10

#define	BUFFER_LEN	1024

char 	sbuf[BUFFER_LEN],databuf[BUFFER_LEN],rbuf[BUFFER_LEN];

int main()
{
	int			sfd, afd;
	struct sockaddr_in 	from, to;
	int 			size,i,ret,len,data_i=0;
	struct termios		tty;

	printf("start\n");
	/* open serial port */
	if ( (afd=open(SPORT, O_RDWR|O_NDELAY)) < 0 ) {
		printf("open asyc port [%s] fail [%d]\n", SPORT, errno);
		return 0;
	}

	/* set serial port baud rate */
	tty.c_cflag = CREAD | CLOCAL | B38400 | CS8;
	tty.c_oflag = 0;
	tty.c_lflag = 0;
	tty.c_iflag = 0;
	tty.c_cc[VMIN] = 0;
	tty.c_cc[VTIME] = 0;
	tcsetattr(afd, TCSANOW, &tty);

	/* create socket */
	if ( (sfd=socket(AF_INET, SOCK_DGRAM, IPPROTO_UDP)) < 0 ){
		printf("open socket fail [%d]\n", errno);
		close(afd);
		return 0;
	}

	from.sin_addr.s_addr = htonl(INADDR_ANY);
	from.sin_family = AF_INET;
	from.sin_port = htons((unsigned short)UDPPORT);
 
	to.sin_addr.s_addr = inet_addr(DEST_IP);
	to.sin_family = AF_INET;
	to.sin_port = htons((unsigned short)UDPPORT);

	/* associates a local address with a socket */
	if ( bind(sfd, (struct sockaddr *)&from, sizeof(from)) < 0 ) {
		printf("socket bind fail [%d]\n", errno);
	    	goto exit_program;
	}

	/* set socket to nonblock */
	ret = fcntl(sfd, F_GETFL);
	ret |= O_NONBLOCK;
	if ( fcntl(sfd, F_SETFL, ret) < 0 ) {
		printf("set socket to nonblock fail [%d] !\n", errno);
		goto exit_program;
	}

	size = sizeof(to);
	while(1){
		/* read data from serial port */
		if ( (len=read(afd,sbuf,BUFFER_LEN)) < 0 ) {
			printf("async port read data fail [%d]\n", errno);
			break;
		}else if(len == 0){
			continue;
		} 
		
		for(i=0; i<len; i++) 
			databuf[data_i++] = sbuf[i];
	            
		for(;;){
			if(data_i != 0){
				/* send data to LAN */		
				if((ret = sendto(sfd,databuf,data_i,0, (struct sockaddr *)&to,size)) < 0 ){
					printf("socket sendto fail [%d]\n", errno);
					goto exit_program;
				}else{
					if(ret == data_i){
						data_i = data_i - ret;
					}else{ 
						data_i = 0;	
						break;
					}
				}
			}else 
				break; 
		}

		/* receive data form LAN */
		ret = recvfrom(sfd, rbuf, BUFFER_LEN, 0, (struct sockaddr *)&from,&size);
		if(ret==0)
			continue;
		else if(ret< 0){
			if (errno == EAGAIN) 
				continue;
			if ( errno == ENOTCONN )
				break;
			printf("socket recvfrom fail [%d]\n", errno);
			break;
		}else{ 
			/* write data to serial port */
			if ( (len=write(afd,rbuf,ret)) < 0 ) {
				printf("async port write data fail [%d]", errno);
				break;
			}
		}
        }		    

exit_program:
	close(sfd);
	close(afd);
	printf("exit\n");
	return 0;
}
